#include "test-ability.h"
#include "test-skill.h"
#include "test-character.h"

#include <adventure-pack.h>

#include <glib.h>
#include <locale.h>

int
main (int    argc,
      char **argv)
{
    /*
     * *************************************************************************
     * Prepare Test
     * *************************************************************************
     */
    setlocale (LC_ALL, "");

    g_test_init (&argc, &argv, NULL);
    g_test_bug_base ("https://github.com/adventure-pack/libadventure-pack/issues");

    /*
     * *************************************************************************
     * Ability Tests
     * *************************************************************************
     */
    g_test_add ("/ability/name", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_name,
                ability_fixture_tear_down);

    g_test_add ("/ability/score", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_score,
                ability_fixture_tear_down);

    g_test_add ("/ability/score-with-bonus", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_score_with_bonus,
                ability_fixture_tear_down);

    for (gint i = 1; i <=  30; i++)
    {
        /*
         * test_name = /ability/modifier/$i
         */
        gchar *test_name = g_strconcat ("/ability/modifier/", g_strdup_printf ("%i", i), NULL);

        g_test_add (test_name, AbilityFixture, GINT_TO_POINTER (i),
                    ability_fixture_set_up,
                    test_ability_modifier,
                    ability_fixture_tear_down);
    }

    g_test_add ("/ability/signal/score-changed", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_signal_score_changed,
                ability_fixture_tear_down);

    g_test_add ("/ability/signal/modifier-changed", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_signal_modifier_changed,
                ability_fixture_tear_down);

    /*
     * *************************************************************************
     * Skill Tests
     * *************************************************************************
     */
    g_test_add ("/skill/name", SkillFixture, NULL,
                skill_fixture_set_up,
                test_skill_name,
                skill_fixture_tear_down);

    g_test_add ("/skill/score", SkillFixture, NULL,
                skill_fixture_set_up,
                test_skill_score,
                skill_fixture_tear_down);

    g_test_add ("/skill/bonus", SkillFixture, NULL,
                skill_fixture_set_up,
                test_skill_bonus,
                skill_fixture_tear_down);

    g_test_add ("/skill/modifier", SkillFixture, NULL,
                skill_fixture_set_up,
                test_skill_modifier,
                skill_fixture_tear_down);

    g_test_add ("/skill/proficient", SkillFixture, NULL,
                skill_fixture_set_up,
                test_skill_proficient,
                skill_fixture_tear_down);

    /*
     * *************************************************************************
     * Character Tests
     * *************************************************************************
     */
    g_test_add ("/character/ability/add", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_ability_add,
                character_fixture_tear_down);

    g_test_add ("/character/ability/add-with-no-name", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_ability_add_with_no_name,
                character_fixture_tear_down);

    g_test_add ("/character/ability/add-with-duplicate-name", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_ability_add_with_duplicate_name,
                character_fixture_tear_down);

    g_test_add ("/character/ability/get", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_ability_get,
                character_fixture_tear_down);

    g_test_add ("/character/skill/add", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_skill_add,
                character_fixture_tear_down);

    g_test_add ("/character/skill/add-with-no-name", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_skill_add_with_no_name,
                character_fixture_tear_down);

    g_test_add ("/character/skill/add-with-duplicate-name", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_skill_add_with_duplicate_name,
                character_fixture_tear_down);

    g_test_add ("/character/skill/get", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_skill_get,
                character_fixture_tear_down);

    g_test_add ("/character/remove/ability", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_remove_ability,
                character_fixture_tear_down);

    g_test_add ("/character/remove/abilities", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_remove_abilities,
                character_fixture_tear_down);

    g_test_add ("/character/remove/skill", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_remove_skill,
                character_fixture_tear_down);

    g_test_add ("/character/remove/skills", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_remove_skills,
                character_fixture_tear_down);

    g_test_add ("/character/remove/all", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_remove_all,
                character_fixture_tear_down);

    g_test_add ("/character/signal/ability-score-changed-skill-score-changed", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_signal_ability_score_changed_skill_score_changed,
                character_fixture_tear_down);

    g_test_add ("/character/signal/skill-modifier-changed", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_signal_skill_modifier_changed,
                character_fixture_tear_down);

    g_test_add ("/character/check/skill-for-lingering-bind-with-ability", CharacterFixture, NULL,
                character_fixture_set_up,
                test_character_check_skill_for_lingering_bind_with_ability,
                character_fixture_tear_down);

    return g_test_run ();
}
